//$Id: AuthorityDaoHibernate.java,v 1.3 2007/01/13 07:06:51 chaostone Exp $
/*
 *
 * Copyright c 2005-2009
 * Licensed under the Apache License, Version 2.0 (the "License")
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 */
/********************************************************************************
 * @author chaostone
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name                 Date                Description 
 * ============         ============        ============
 * chaostone             2005-9-27         Created
 *  
 ********************************************************************************/

package org.beangle.security.internal.dao;

import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.beangle.commons.collection.MapUtil;
import org.beangle.model.persist.impl.AbstractBaseDao;
import org.beangle.model.query.builder.OqlBuilder;
import org.beangle.security.Authority;
import org.beangle.security.Group;
import org.beangle.security.menu.Menu;
import org.beangle.security.menu.MenuAuthority;
import org.beangle.security.menu.MenuProfile;
import org.beangle.security.menu.dao.MenuAuthorityDao;

public class MenuAuthorityDaoImpl extends AbstractBaseDao implements MenuAuthorityDao {

	public MenuAuthority getMenuAuthority(Group group, Menu menu) {
		Map<String, Object> params = MapUtil.newHashMap();
		params.put("group", group);
		params.put("menu", menu);
		List<?> authorityList = entityDao.searchNamedQuery("getGroupMenu", params, false);
		if (authorityList.isEmpty())
			return null;
		else
			return (MenuAuthority) authorityList.get(0);
	}

	@SuppressWarnings("unchecked")
	public List<MenuAuthority> getMenuAuthorities(MenuProfile profile, Group group, int depth,
			String ancestorCode) {
		StringBuilder hql = new StringBuilder(" select a  from MenuAuthority a join a.group as r");
		hql.append(" join a.menu as m where r = :group" + " and m.enabled=true ");
		if (null != profile) {
			hql.append(" and m.profile=:profile");
		}
		if (StringUtils.isNotEmpty(ancestorCode)) {
			hql.append(" and m.code like :ancestorCodePattern ").append(
					"and length(m.code)>" + ancestorCode.length());
		}
		if (depth > 0) {
			hql.append(" and length(m.code)/2 <= ").append(depth);
		}
		OqlBuilder query = OqlBuilder.hql(hql.toString());
		query.param("group", group);

		if (StringUtils.isNotEmpty(ancestorCode)) {
			query.param("ancestorCodePattern", ancestorCode + "%");
		}
		if (depth > 0) {
			query.param("depth", new Long(depth));
		}
		if (null != profile) {
			query.param("profile", profile);
		}
		return (List<MenuAuthority>) entityDao.search(query);
	}

	/**
	 * 查询用户组对应的模块
	 */
	@SuppressWarnings("unchecked")
	public List<Menu> getMenus(MenuProfile profile, Group group, int depth, String ancestorCode) {
		OqlBuilder query = buildMenuQuery(profile, group, depth, ancestorCode);
		query.where("m.enabled=true").cacheable();
		return entityDao.search(query);
	}

	public List<Menu> getAllMenus(MenuProfile profile, Group group, int depth, String ancestorCode) {
		return entityDao.search(buildMenuQuery(profile, group, depth, ancestorCode));
	}

	private OqlBuilder<Menu> buildMenuQuery(MenuProfile profile, Group group, int depth,
			String ancestorCode) {
		StringBuilder hql = new StringBuilder(" select m from MenuAuthority a join a.group as r");
		hql.append(" join a.menu as m where r = :group");
		if (null != profile) {
			hql.append(" and m.profile=:profile");
		}
		if (StringUtils.isNotEmpty(ancestorCode)) {
			hql.append(" and m.code like :ancestorCodePattern").append(
					" and length(m.code)>" + ancestorCode.length());
		}
		if (depth > 0) {
			hql.append(" and length(m.code)/2 <= ").append(depth);
		}
		OqlBuilder<Menu> query = OqlBuilder.hql(hql.toString());
		query.param("group", group);

		if (StringUtils.isNotEmpty(ancestorCode)) {
			query.param("ancestorCodePattern", ancestorCode + "%");
		}
		if (null != profile) {
			query.param("profile", profile);
		}
		return query;
	}

	public void saveOrUpdate(Authority authority) {
		entityDao.saveOrUpdate(authority);
	}

	public void remove(Authority authority) {
		entityDao.remove(authority);
	}

}
