//$Id: AuthorityServiceImpl.java,v 1.6 2007/01/22 13:01:24 chaostone Exp $
/*
 *
 * Copyright c 2005-2009
 * Licensed under the Apache License, Version 2.0 (the "License")
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 */
/********************************************************************************
 * @author chaostone
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name                 Date                Description 
 * ============         ============        ============
 * dell                                     Created
 * chaostone            2005-9-26           rename some method's signature
 *  
 ********************************************************************************/

package org.beangle.security.internal.service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.beangle.commons.collection.ListUtil;
import org.beangle.commons.collection.MapUtil;
import org.beangle.commons.collection.SetUtil;
import org.beangle.model.entity.Model;
import org.beangle.model.persist.impl.BaseServiceImpl;
import org.beangle.security.Authority;
import org.beangle.security.Group;
import org.beangle.security.Resource;
import org.beangle.security.User;
import org.beangle.security.dao.AuthorityDao;
import org.beangle.security.service.AuthorityService;
import org.beangle.security.service.ResourceService;
import org.beangle.security.service.UserService;

/**
 * 授权信息的服务实现类
 * 
 * @author dell,chaostone 2005-9-26
 */
public class AuthorityServiceImpl extends BaseServiceImpl implements AuthorityService {

	protected AuthorityDao authorityDao;

	protected UserService userService;

	protected ResourceService resourceService;

	public Resource getResource(String name) {
		return resourceService.getResource(name);
	}

	public List<Authority> getAuthorities(User user) {
		if (null == user)
			return Collections.emptyList();
		Map<Resource, Authority> authorities = MapUtil.newHashMap();
		if (null != user.getGroups()) {
			for (final Group group : user.getAllGroups()) {
				List<Authority> groupAuths = authorityDao.getAuthorities(group);
				for (final Authority groupAuth : groupAuths) {
					if (authorities.containsKey(groupAuth.getResource())) {
						Authority existed = authorities.get(groupAuth.getResource());
						existed.merge(groupAuth);
					} else {
						authorities.put(groupAuth.getResource(), groupAuth);
					}
				}
			}
		}
		return ListUtil.newArrayList(authorities.values());
	}

	public Authority getAuthority(User user, Resource resource) {
		if ((null == user) || null == resource)
			return null;
		Authority au = null;
		Set<Group> groups = user.getAllGroups();
		if (null != groups)
			for (final Group one : groups) {
				Authority ar = getAuthority(one, resource);
				if (null == au) {
					au = ar;
				} else {
					au.merge(ar);
				}
			}
		return au;
	}

	@SuppressWarnings("unchecked")
	public List<Resource> getResources(User user) {
		Set<Resource> resources = SetUtil.newHashSet();
		Map<String, Object> params = MapUtil.newHashMap();
		Set<Group> groups = user.getAllGroups();
		String hql = "select distinct m from Group as r join r.authorities as a"
				+ " join a.resource as m where  r.id = :groupId";
		params.clear();
		for (final Group group : groups) {
			params.put("groupId", group.getId());
			resources.addAll(entityDao.searchHQLQuery(hql, params));
		}
		return ListUtil.newArrayList(resources);
	}

	public List<Authority> getAuthorities(Group group) {
		return authorityDao.getAuthorities(group);
	}

	public Authority getAuthority(Group group, Resource resource) {
		if (null == group || null == resource)
			return null;
		else
			return authorityDao.getAuthority(group, resource);
	}

	public List<Resource> getResources(Group group) {
		return authorityDao.getResources(group);
	}

	public Set<Long> getResourceIds(Group group) {
		return authorityDao.getResourceIds(group);
	}

	public void authorize(Group group, Set<Resource> resources) {
		// 查找保留的权限
		Set<Authority> reserved = SetUtil.newHashSet();
		for (final Authority authority : group.getAuthorities()) {
			if (resources.contains(authority.getResource())) {
				reserved.add(authority);
				resources.remove(authority.getResource());
			}
		}
		group.getAuthorities().clear();
		group.getAuthorities().addAll(reserved);
		// 新权限
		Authority model = null;
		try {
			model = (Authority) Model.newInstance(Authority.class);
		} catch (Exception e) {
			throw new RuntimeException("cannot init authroity by class:" + Authority.class);
		}
		model.setGroup(group);
		for (final Resource element : resources) {
			Authority authority = (Authority) model.clone();
			authority.setResource(element);
			group.getAuthorities().add(authority);
		}
		entityService.saveOrUpdate(group);
	}

	public void remove(Authority authority) {
		if (null != authority)
			entityService.remove(authority);
	}

	public void saveOrUpdate(Authority authority) {
		entityService.saveOrUpdate(authority);
	}

	public void setAuthorityDao(AuthorityDao authorityDao) {
		this.authorityDao = authorityDao;
	}

	public void setResourceService(ResourceService resourceService) {
		this.resourceService = resourceService;
	}

	public void setUserService(UserService userService) {
		this.userService = userService;
	}

	public UserService getUserService() {
		return userService;
	}
}
