//$Id: AuthorityServiceImpl.java,v 1.6 2007/01/22 13:01:24 chaostone Exp $
/*
 *
 * Copyright c 2005-2009
 * Licensed under the Apache License, Version 2.0 (the "License")
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 */
/********************************************************************************
 * @author chaostone
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name                 Date                Description 
 * ============         ============        ============
 * dell                                     Created
 * chaostone            2005-9-26           rename some method's signature
 *  
 ********************************************************************************/

package org.beangle.security.internal.service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.commons.collections.CollectionUtils;
import org.beangle.commons.collection.ListUtil;
import org.beangle.commons.collection.SetUtil;
import org.beangle.model.entity.Model;
import org.beangle.model.persist.impl.BaseServiceImpl;
import org.beangle.security.Authority;
import org.beangle.security.Group;
import org.beangle.security.Resource;
import org.beangle.security.User;
import org.beangle.security.menu.Menu;
import org.beangle.security.menu.MenuAuthority;
import org.beangle.security.menu.MenuProfile;
import org.beangle.security.menu.dao.MenuAuthorityDao;
import org.beangle.security.menu.service.MenuAuthorityService;
import org.beangle.security.service.UserService;

/**
 * 授权信息的服务实现类
 * 
 * @author dell,chaostone 2005-9-26
 */
public class MenuAuthorityServiceImpl extends BaseServiceImpl implements MenuAuthorityService {

	protected MenuAuthorityDao menuAuthorityDao;

	protected UserService userService;

	public List<MenuAuthority> getMenuAuthorities(MenuProfile profile, User user) {
		return getMenuAuthorities(profile, user, -1, null);
	}

	public List<MenuAuthority> getMenuAuthorities(MenuProfile profile, User user, int depth,
			String ancestorCode) {
		if (null == user)
			return Collections.emptyList();
		Set<MenuAuthority> menuAuthorities = SetUtil.newHashSet();
		if (null != user.getGroups()) {
			for (Group g : user.getGroups()) {
				menuAuthorities.addAll(menuAuthorityDao.getMenuAuthorities(profile, g, depth,
						ancestorCode));
			}
		}
		List<MenuAuthority> authorityList = new ArrayList<MenuAuthority>(menuAuthorities);
		Collections.sort(authorityList);
		return authorityList;
	}

	public List<MenuAuthority> getMenuAuthorities(MenuProfile profile, Group group) {
		return getMenuAuthorities(profile, group, -1);
	}

	public List<MenuAuthority> getMenuAuthorities(MenuProfile profile, Group group, int depth) {
		return getMenuAuthorities(profile, group, depth, null);
	}

	public List<MenuAuthority> getMenuAuthorities(MenuProfile profile, Group group, int depth,
			String ancestorCode) {
		if (null == group)
			return Collections.emptyList();
		return menuAuthorityDao.getMenuAuthorities(profile, group, depth, ancestorCode);
	}

	public MenuAuthority getMenuAuthority(Group group, Menu menu) {
		if (group == null || null == menu)
			return null;
		else
			return menuAuthorityDao.getMenuAuthority(group, menu);
	}

	public List<Menu> getMenus(MenuProfile profile, User user) {
		return getMenus(profile, user, -1, null);
	}

	public List<Menu> getMenus(MenuProfile profile, User user, int depth, String ancestorCode) {
		Set<Menu> modules = SetUtil.newHashSet();
		Set<Group> groups = userService.getGroups(user);
		for (final Group group : groups) {
			modules.addAll(menuAuthorityDao.getMenus(profile, group, depth, ancestorCode));
		}
		if (null == modules)
			return Collections.emptyList();
		List<Menu> moduleList = ListUtil.newArrayList(modules);
		Collections.sort(moduleList);
		return moduleList;
	}

	public List<Menu> getMenus(MenuProfile profile, Group group) {
		return menuAuthorityDao.getMenus(profile, group, 0, null);
	}

	public List<Menu> getAllMenus(MenuProfile profile, Group group) {
		return menuAuthorityDao.getAllMenus(profile, group, 0, null);
	}

	/**
	 * @deprecated
	 */
	@SuppressWarnings("unchecked")
	public void copyAuthority(MenuProfile profile, Group fromGroup, Collection toGroups) {
		List fromAuthorities = getMenuAuthorities(profile, fromGroup);
		List allAdded = ListUtil.newArrayList();
		for (Iterator iter = toGroups.iterator(); iter.hasNext();) {
			Group toGroup = (Group) iter.next();
			List toAuthorities = getMenuAuthorities(profile, toGroup);
			Collection newAuthorities = CollectionUtils.subtract(fromAuthorities, toAuthorities);
			for (Iterator iterator = newAuthorities.iterator(); iterator.hasNext();) {
				// GroupAuthority auth = (GroupAuthority) iterator.next();
				// allAdded.add(auth.clone());
			}
		}
		entityService.saveOrUpdate(allAdded);
	}

	public void authorize(Group group, Set<Resource> resources) {
		// 查找保留的权限
		Set<Authority> reserved = SetUtil.newHashSet();
		for (final Authority authority:group.getAuthorities()){
			if (resources.contains(authority.getResource())) {
				reserved.add(authority);
				resources.remove(authority.getResource());
			}
		}
		group.getAuthorities().clear();
		group.getAuthorities().addAll(reserved);
		// 新权限
		Authority model = null;
		try {
			model = (Authority) Model.newInstance(Authority.class);
		} catch (Exception e) {
			throw new RuntimeException("cannot init authroity by class:" + Authority.class);
		}
		model.setGroup(group);
		for (final Resource element: resources){
			Authority authority =(Authority) model.clone();
			authority.setResource(element);
			group.getAuthorities().add(authority);
		}
		entityService.saveOrUpdate(group);
	}

	public void saveOrUpdate(MenuAuthority o) {
		entityService.saveOrUpdate(o);
	}

	public void setMenuAuthorityDao(MenuAuthorityDao menuAuthorityDao) {
		this.menuAuthorityDao = menuAuthorityDao;
	}

	public void setUserService(UserService userService) {
		this.userService = userService;
	}

}
