//$Id: RestrictionEntityQueryApply.java 2008-6-9 下午09:38:33 chaostone Exp $
/*
 *
 * Copyright c 2005-2009
 * Licensed under the Apache License, Version 2.0 (the "License")
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * 
 */
/********************************************************************************
 * @author chaostone
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name           Date          Description 
 * ============   ============  ============
 * chaostone      2008-6-9  Created
 *  
 ********************************************************************************/
package org.beangle.security.internal.service;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.beangle.commons.collection.ListUtil;
import org.beangle.commons.collection.MapUtil;
import org.beangle.model.query.builder.Condition;
import org.beangle.model.query.builder.OqlBuilder;
import org.beangle.security.AuthorityException;
import org.beangle.security.restriction.Param;
import org.beangle.security.restriction.ParamGroup;
import org.beangle.security.restriction.Pattern;
import org.beangle.security.restriction.Restriction;
import org.beangle.security.restriction.service.RestrictionApply;

public class RestrictionEntityQueryApply implements RestrictionApply {

	public void apply(OqlBuilder<?> query, Collection<? extends Pattern> patterns,
			Collection<? extends Restriction> restrictions) {
		String prefix = "(";
		StringBuilder conBuffer = new StringBuilder();
		List<Object> paramValues = ListUtil.newArrayList();
		int index = 0;
		Map<ParamGroup, Pattern> patternMap = MapUtil.newHashMap();
		for (final Pattern pattern : patterns) {
			patternMap.put(pattern.getParamGroup(), pattern);
		}

		for (final Restriction restriction : restrictions) {
			// 处理限制对应的模式
			Pattern pattern = (Pattern) patternMap.get(restriction.getParamGroup());
			if (null == pattern || StringUtils.isEmpty(pattern.getContent())) {
				continue;
			}
			String patternContent = pattern.getContent();
			patternContent = StringUtils.replace(patternContent, "{alias}", query.getAlias());
			String[] contents = StringUtils.split(
					StringUtils.replace(patternContent, " and ", "$"), "$");

			StringBuilder singleConBuf = new StringBuilder(prefix);
			for (int i = 0; i < contents.length; i++) {
				String content = contents[i];
				Condition c = new Condition(content);
				List<String> params = c.getParamNames();
				for (final String paramName : params) {
					Param param = pattern.getParamGroup().getParam(paramName);
					String value = restriction.getItem(param);
					if (StringUtils.isNotEmpty(value)) {
						if (value.equals(Restriction.ALL)) {
							content = "";
						} else {
							content = StringUtils.replace(content, ":" + param.getName(), ":"
									+ param.getName() + index);
							paramValues.add(restriction.getValue(param));
						}
					} else {
						throw new AuthorityException(paramName + " had not been initialized");
					}
				}
				if (singleConBuf.length() > 1 && content.length() > 0) {
					singleConBuf.append(" and ");
				}
				singleConBuf.append(content);
			}

			if (singleConBuf.length() > 1) {
				singleConBuf.append(')');
				if (conBuffer.length() > 0) {
					conBuffer.append(" or ");
				}
				conBuffer.append(singleConBuf.toString());
			}
			index++;
		}
		if (conBuffer.length() > 0) {
			Condition con = new Condition(conBuffer.toString());
			con.params(paramValues);
			query.where(con);
		}
	}

	public void apply(OqlBuilder<?> query, Pattern pattern, Restriction restriction) {
		apply(query, Collections.singletonList(pattern), Collections.singletonList(restriction));
	}

}
