//$Id: RetrictionServiceImpl.java,v 1.1 2007-10-14 下午04:41:35 chaostone Exp $
/*
 *
 * Copyright c 2005-2009
 * Licensed under the Apache License, Version 2.0 (the "License")
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * 
 */
/********************************************************************************
 * @author chaostone
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name                 Date                Description 
 * ============         ============        ============
 * chaostone              2007-10-14         Created
 *  
 ********************************************************************************/

package org.beangle.security.internal.service;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.Predicate;
import org.apache.commons.lang.StringUtils;
import org.beangle.commons.collection.ListUtil;
import org.beangle.commons.collection.MapUtil;
import org.beangle.commons.collection.SetUtil;
import org.beangle.model.persist.impl.BaseServiceImpl;
import org.beangle.model.query.builder.OqlBuilder;
import org.beangle.security.Resource;
import org.beangle.security.User;
import org.beangle.security.restriction.Param;
import org.beangle.security.restriction.ParamGroup;
import org.beangle.security.restriction.Pattern;
import org.beangle.security.restriction.Restriction;
import org.beangle.security.restriction.RestrictionHolder;
import org.beangle.security.restriction.service.RestrictionApply;
import org.beangle.security.restriction.service.RestrictionService;

public class RestrictionServiceImpl extends BaseServiceImpl implements RestrictionService {

	private RestrictionApply restrictionApply;

	/**
	 * 查询用户在指定模块的数据权限
	 */
	@SuppressWarnings("unchecked")
	public List<Restriction> getRestrictions(final User user, final Resource resource) {
		List<Restriction> restrictions = ListUtil.newArrayList();
		final Set<ParamGroup> paramGroups = SetUtil.newHashSet();
		for (final Pattern pattern : resource.getPatterns()) {
			paramGroups.add(pattern.getParamGroup());
		}
		// 权限上的限制
		restrictions.addAll(getAuthorityRestrictions(user, resource));
		// 用户组自身限制
		for (RestrictionHolder group : user.getGroups()) {
			restrictions.addAll(group.getRestrictions());
		}
		// 用户自身限制
		RestrictionHolder userHolder = (RestrictionHolder) user;
		restrictions.addAll(userHolder.getRestrictions());
		// 模式过滤
		return (List) CollectionUtils.select(restrictions, new Predicate() {
			public boolean evaluate(Object obj) {
				Restriction restriciton = (Restriction) obj;
				if (restriciton.isEnabled() && paramGroups.contains(restriciton.getParamGroup()))
					return true;
				else
					return false;
			}
		});
	}

	@SuppressWarnings("unchecked")
	public List<Restriction> getAuthorityRestrictions(User user, Resource resource) {
		OqlBuilder query = OqlBuilder.hql("select restriction from Authority r "
				+ "join r.group.users as user join r.restrictions as restriction"
				+ " where user=:user and r.resource=:resource" + " and restriction.enabled=true");
		Map<String, Object> params = MapUtil.newHashMap();
		params.put("user", user);
		params.put("resource", resource);
		query.params(params);
		return entityService.search(query);
	}

	@SuppressWarnings("unchecked")
	public List getValues(Param param) {
		if (null == param.getEditor())
			return Collections.emptyList();
		OqlBuilder query = OqlBuilder.hql(param.getEditor().getSource());
		List rs = entityService.search(query);
		logger.debug("param size {},source:{} ", new Integer(rs.size()), param.getEditor()
				.getSource());
		return rs;
	}

	public <T> Set<T> select(Collection<T> values, List<? extends Restriction> restrictions,
			Param param) {
		Set<T> selected = SetUtil.newHashSet();
		for (final Restriction restriction : restrictions) {
			selected.addAll(select(values, restriction, param));
		}
		return selected;
	}

	public <T> Set<T> select(Collection<T> values, final Restriction restriction, Param param) {
		Set<T> selected = SetUtil.newHashSet();
		String value = restriction.getItem(param);
		if (StringUtils.isNotEmpty(value)) {
			if (value.equals(Restriction.ALL)) {
				selected.addAll(values);
				return selected;
			}
			@SuppressWarnings("unchecked")
			final Set<T> paramValue = (Set<T>) restriction.getValue(param);
			for (T obj : values) {
				try {
					if (paramValue.contains(PropertyUtils.getProperty(obj, param.getEditor()
							.getIdProperty()))) {
						selected.add(obj);
					}
				} catch (Exception e) {
					throw new RuntimeException(e.getMessage());
				}
			}
		}
		return selected;
	}

	public void setRestrictionApply(RestrictionApply restrictionApply) {
		this.restrictionApply = restrictionApply;
	}

	public void apply(OqlBuilder<?> builder, Collection<? extends Pattern> patterns,
			Collection<? extends Restriction> restrictions) {
		restrictionApply.apply(builder, patterns, restrictions);
	}

	public void apply(OqlBuilder<?> builder, Pattern pattern, Restriction restriction) {
		restrictionApply.apply(builder, pattern, restriction);
	}
}
