//$Id: UserServiceImpl.java,v 1.2 2006/10/19 09:21:32 chaostone Exp $
/*
 *
 * Copyright c 2005-2009
 * Licensed under the Apache License, Version 2.0 (the "License")
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 */
/********************************************************************************
 * @author chaostone
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name                 Date                Description 
 * ============         ============        ============
 * dell                                     Created
 * chaostone            2005-9-26           rename some method's signature         
 *  
 ********************************************************************************/

package org.beangle.security.internal.service;

import java.sql.Date;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.beangle.commons.collection.SetUtil;
import org.beangle.model.EntityExistException;
import org.beangle.model.persist.impl.BaseServiceImpl;
import org.beangle.model.query.builder.OqlBuilder;
import org.beangle.security.AdminUser;
import org.beangle.security.Group;
import org.beangle.security.User;
import org.beangle.security.dao.UserDao;
import org.beangle.security.service.UserService;
import org.springframework.dao.DataIntegrityViolationException;

/**
 * 用户信息服务的实现类
 * 
 * @author dell,chaostone 2005-9-26
 */
public class UserServiceImpl extends BaseServiceImpl implements UserService {

	private UserDao userDao;

	public void setUserDao(UserDao userDao) {
		this.userDao = userDao;
	}

	public boolean isAdmin(User user) {
		OqlBuilder<?> query = OqlBuilder.from(AdminUser.class, "admin");
		query.where("admin.user=:user", user).select("admin.id");
		return !entityService.search(query).isEmpty();
	}
 
	public User get(String name, String password) {
		return userDao.get(name, password);
	}

	public User get(Long id) {
		return userDao.get(id);
	}

	public User get(String loginName) {
		if (StringUtils.isEmpty(loginName))
			return null;
		OqlBuilder<User> entityQuery = OqlBuilder.from(User.class, "user");
		entityQuery.where("user.name=:name", loginName);
		List<User> users = entityService.search(entityQuery);
		return (users.isEmpty()) ? null : users.get(0);
	}

	public List<User> getUsers(Long[] userIds) {
		return userDao.get(userIds);
	}

	public void updateState(Long[] ids, int state) {
		assert (null == ids || ids.length < 1);
		List<User> users = userDao.get(ids);
		for (int i = 0; i < users.size(); i++) {
			User cur = users.get(i);
			cur.setStatus(state);
		}
		entityService.saveOrUpdate(users);
	}

	public void saveOrUpdate(User user) throws EntityExistException {
		try {
			user.setUpdatedAt(new Date(System.currentTimeMillis()));
			if (!user.saved()) {
				user.setCreatedAt(new Date(System.currentTimeMillis()));
			}
			entityService.saveOrUpdate(user);
		} catch (DataIntegrityViolationException e) {
			throw new EntityExistException("User already exits:" + user);
		} catch (Exception e) {
			throw new EntityExistException("User already exits:" + user);
		}
	}

	public Set<Group> getGroups(User user) {
		//workgroup for no session 
		OqlBuilder<Group> builder =OqlBuilder.from(Group.class,"g");
		builder.join("g.users", "u").where("u=:user",user);
		return SetUtil.newHashSet(entityDao.search(builder));
	}

	public void createUser(User creator, User newUser) {
		newUser.setCreator(creator);
		newUser.setUpdatedAt(new Date(System.currentTimeMillis()));
		newUser.setCreatedAt(new Date(System.currentTimeMillis()));
		entityDao.saveOrUpdate(newUser);
	}

	public void removeUser(User manager, User user) {
		if (isManagedBy(manager, user)) {
			entityService.remove(user);
		}
	}

	public boolean isManagedBy(User manager, User user) {
		if (isAdmin(manager) || manager.equals(user.getCreator())) {
			return true;
		} else {
			for (final Group group : manager.getMngGroups()) {
				if (group.getUsers().contains(user)) {
					return true;
				}
			}
			return false;
		}
	}

	public void createGroup(User creator, Group group) {
		group.setUpdatedAt(new Date(System.currentTimeMillis()));
		group.setCreatedAt(new Date(System.currentTimeMillis()));
		group.setCreator(creator);
		// 纳入创建者的管理范围
		creator.getMngGroups().add(group);
		entityDao.saveOrUpdate(group);
		entityDao.saveOrUpdate(creator);
	}

	/**
	 * 超级管理员不能删除
	 */
	public void removeGroup(User manager, List<Group> groups) {
		for (final Group group : groups) {
			if (manager.getMngGroups().contains(group))
				manager.getMngGroups().remove(group);
			if (group.getCreator().equals(manager) || isAdmin(manager))
				entityService.remove(group);
		}
		entityService.saveOrUpdate(manager);
	}

}
