//$Id: User.java,v 1.7 2007/01/05 03:48:05 chaostone Exp $
/*
 *
 * Copyright c 2005-2009
 * Licensed under the Apache License, Version 2.0 (the "License")
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 */
/********************************************************************************
 * @author chaostone
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name                 Date                Description 
 * ============         ============        ============
 * dell                 2005-9-15           Created
 * chaostone            2005-9-26           add mngUsers
 *  
 ********************************************************************************/

package org.beangle.security.model;

import java.util.Set;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.beangle.commons.collection.SetUtil;
import org.beangle.model.pojo.LongIdTimeObject;
import org.beangle.security.Group;
import org.beangle.security.User;
import org.beangle.security.UserCategory;
import org.beangle.security.restriction.Restriction;
import org.beangle.security.restriction.RestrictionHolder;

/**
 * 系统中所有用户的账号、权限、状态信息.
 * 
 * @author dell,chaostone 2005-9-26
 */
public class UserBean extends LongIdTimeObject implements User, RestrictionHolder {
	private static final long serialVersionUID = -3625902334772342380L;

	/** 名称 */
	protected String name;

	/** 用户姓名 */
	private String fullname;

	/** 用户密文 */
	private String password;

	/** 用户联系email */
	private String mail;

	/** 对应用户组 */
	private Set<Group> groups = SetUtil.newHashSet();

	/** 创建人 */
	private User creator;

	/** 向下级授权,所管理的用户组 */
	private Set<Group> mngGroups = SetUtil.newHashSet();

	/** 种类 */
	protected Set<UserCategory> categories = SetUtil.newHashSet();;

	/** 缺省类别 */
	private UserCategory defaultCategory;

	/** 状态 */
	protected int status = User.ACTIVE;

	/** 访问限制 */
	protected Set<Restriction> restrictions = SetUtil.newHashSet();

	/** 备注 */
	protected String remark;

	public UserBean() {
		super();
	}

	public UserBean(Long id) {
		setId(id);
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getRemark() {
		return remark;
	}

	public void setRemark(String remark) {
		this.remark = remark;
	}

	public String getMail() {
		return mail;
	}

	public void setMail(String mail) {
		this.mail = mail;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public boolean isCategory(Long categoryId) {
		for (final UserCategory category : categories) {
			if (category.getId().equals(categoryId))
				return true;
		}
		return false;
	}

	public Set<Group> getAllGroups() {
		Set<Group> allGroups = SetUtil.newHashSet();
		for (Group group : groups) {
			while (null != group) {
				allGroups.add(group);
				group = group.getParent();
			}
		}
		return allGroups;
	}

	public String getFullname() {
		return fullname;
	}

	public void setFullname(String fullname) {
		this.fullname = fullname;
	}

	public User getCreator() {
		return creator;
	}

	public void setCreator(User creator) {
		this.creator = creator;
	}

	public UserCategory getDefaultCategory() {
		return defaultCategory;
	}

	public void setDefaultCategory(UserCategory defaultCategory) {
		this.defaultCategory = defaultCategory;
	}

	public int getStatus() {
		return status;
	}

	public void setStatus(int status) {
		this.status = status;
	}

	/**
	 * 1 激活
	 */
	public boolean isEnabled() {
		return status == 1;
	}

	public Set<Group> getGroups() {
		return groups;
	}

	public void setGroups(Set<Group> groups) {
		this.groups = groups;
	}

	public Set<Group> getMngGroups() {
		return mngGroups;
	}

	public void setMngGroups(Set<Group> mngGroups) {
		this.mngGroups = mngGroups;
	}

	public Set<UserCategory> getCategories() {
		return categories;
	}

	public void setCategories(Set<UserCategory> categories) {
		this.categories = categories;
	}

	public Set<Restriction> getRestrictions() {
		return restrictions;
	}

	public void setRestrictions(Set<Restriction> restrictions) {
		this.restrictions = restrictions;
	}

	public String toString() {
		return new ToStringBuilder(this).append("id", this.id).append("password", this.password)
				.append("name", this.getName()).toString();
	}
}
